import {
  Component,
  Show,
  createEffect,
  createSelector,
  createSignal,
  createUniqueId,
  onMount,
} from "solid-js";
import cards from "../material/cards.module.css";
import TextField from "../material/TextField.js";
import Button from "../material/Button.js";
import { useDocumentTitle } from "../utils";
import { Title } from "../material/typography";
import { css } from "solid-styled";
import { LinearProgress } from "@suid/material";
import { createRestAPIClient } from "masto";
import { getOrRegisterApp } from "./stores";
import { useSearchParams } from "@solidjs/router";
import { $settings } from "../settings/stores";

type ErrorParams = {
  error: string;
  errorDescription: string;
};

const SignIn: Component = () => {
  const progressId = createUniqueId();
  const [params] = useSearchParams<ErrorParams>();
  const [rawServerUrl, setRawServerUrl] = createSignal("");
  const [currentState, setCurrentState] = createSignal<
    "inactive" | "contracting" | "navigating"
  >("inactive");
  const [serverUrlHelperText, setServerUrlHelperText] = createSignal("");
  const [serverUrlError, setServerUrlError] = createSignal(false);
  const [targetSiteTitle, setTargetSiteTitle] = createSignal("");

  useDocumentTitle("Sign In");
  css`
    form {
      display: flex;
      flex-flow: column;
      gap: 16px;
    }
  `;

  const serverUrl = () => {
    const url = rawServerUrl();
    if (url.length === 0 || /^%w:/.test(url)) {
      return url;
    } else {
      return `https://${url}`;
    }
  };

  createEffect(() => {
    const url = serverUrl();
    setServerUrlError(false);
    if (url.length === 0) {
      return;
    }
    try {
      new URL(url);
    } catch {
      setServerUrlHelperText("Domain is required.");
      return;
    }
    setServerUrlHelperText("");
  });

  onMount(() => {
    $settings.setKey("onGoingOAuth2Process", undefined);
  });

  const onStartOAuth2 = async (e: Event) => {
    e.preventDefault();
    setCurrentState("contracting");
    const url = serverUrl();
    try {
      setServerUrlError(!!serverUrlHelperText());

      const client = createRestAPIClient({
        url,
      });
      const ins = await client.v2.instance.fetch();
      setTargetSiteTitle(ins.title);

      const redirectURL = new URL(
        "./oauth2/mastodon",
        window.location.href,
      ).toString();

      const app = await getOrRegisterApp(url, redirectURL);
      if (app === null) {
        alert("The mastodon server could not be used with tutu.");
        return;
      }

      const authStart = new URL("./oauth/authorize", url);
      const searches = authStart.searchParams;
      const args = {
        response_type: "code",
        client_id: app.clientId,
        redirect_uri: redirectURL,
        scope: "read write push",
      };
      for (const [k, v] of Object.entries(args)) {
        searches.set(k, v);
      }
      $settings.setKey("onGoingOAuth2Process", url);
      window.location.href = authStart.toString();
    } catch (e) {
      setServerUrlHelperText(
        `Could not contract with the server: "${String(e)}". Please check and try again.`,
      );
      setServerUrlError(true);
      console.error(`Failed to contract ${url}.`, e);
    } finally {
      setCurrentState("inactive");
    }
  };

  return (
    <div class={cards.layoutCentered}>
      <Show when={params.error || params.errorDescription}>
        <div class={cards.card} style={{ "margin-bottom": "20px" }}>
          <p>Authorization is failed.</p>
          <p>{params.errorDescription}</p>
          <p>
            Please try again later. If the problem persist, you can seek for
            help from the server administrator.
          </p>
        </div>
      </Show>
      <div
        class={/*once*/ cards.card}
        aria-busy={currentState() !== "inactive" ? "true" : "false"}
        aria-describedby={
          currentState() !== "inactive" ? progressId : undefined
        }
        style={{
          padding: `var(--safe-area-inset-top) var(--safe-area-inset-right) var(--safe-area-inset-bottom) var(--safe-area-inset-left)`,
        }}
      >
        <LinearProgress
          class={[cards.cardNoPad, cards.cardGutSkip].join(" ")}
          id={progressId}
          sx={currentState() === "inactive" ? { display: "none" } : undefined}
        />
        <form onSubmit={onStartOAuth2}>
          <Title component="h6">Sign in with Your Mastodon Account</Title>
          <TextField
            label="Mastodon Server"
            name="serverUrl"
            onInput={setRawServerUrl}
            required
            helperText={serverUrlHelperText()}
            error={!!serverUrlError()}
          />

          <div style={{ display: "flex", "justify-content": "end" }}>
            <Button type="submit" disabled={currentState() !== "inactive"}>
              {currentState() == "inactive"
                ? "Continue"
                : currentState() == "contracting"
                  ? `Contracting ${new URL(serverUrl()).host}...`
                  : `Moving to ${targetSiteTitle}`}
            </Button>
          </div>
        </form>
      </div>
    </div>
  );
};

export default SignIn;
